//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 15/12/01; 13:41:51

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "menulib.h"
#include "kblib.h"
#include "dart.h"

//--------------------------------------------------------------------------


extern TMenu * Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//--------------------------------------------------------------------------

extern KBSTATE kb_state;
extern KBSTATE kb_state_DOWN;
extern KBSTATE kb_state_UP;
extern void (*HandlerFuncPtr5)(void);

//--------------------------------------------------------------------------

unsigned char Image_Cursor[] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

//--------------------------------------------------------------------------

char * Plane0;
char * Plane1;

char * SpeedSprite;

TGame * Game;

int PointsList[TARGET_PARTS] = {6,13,4,18,1,20,5,12,9,14,11,8,16,7,19,3,17,2,15,10};

enum {S_BULLSEYE,S_BULL,S_DOUBLE,S_SIMPLE,S_MISSED};
char *SpecialStr[] = {"BULL'S EYE!","BULL!","DOUBLE!","","MISSED!"};

//--------------------------------------------------------------------------

void DrawTarget(void)
{
	
	PortSet(Plane0,239,127);
	
	
	SCR_RECT*SR=&(SCR_RECT){{0, 0, 159, 99}};
	
	ClrScr();
	FontSetSys(0);
	
	// Target
	
	DrawClipEllipse(50,50,50,50,SR,A_NORMAL);
	DrawClipEllipse(50,50,30,30,SR,A_NORMAL);
	
	int a;
	for (a=0;a<TARGET_PARTS;a++)
	{
		
		float angle=(2*PI)*((float)a/TARGET_PARTS);
		
		int line_x=50*cos(angle);
		int line_y=50*sin(angle);
		
		DrawLine(50,50,50+line_x,50-line_y,A_NORMAL);
		
		angle+=((2*PI)/(2*TARGET_PARTS));
		int num_x=42*cos(angle);
		int num_y=42*sin(angle);
		
		
		char buffer[8];
		sprintf(buffer,"%d",PointsList[a]);
		DrawStr((50+num_x)-2,(50-num_y)-3,buffer,A_NORMAL);
		
	}
	
	// Speed Column
	
	DrawClipRect (&(WIN_RECT){104,0,111,99},SR,A_NORMAL);
	
	FontSetSys(1);
	
	PortRestore();
	FastCopyScreen(Plane0,Plane1);
	//ngetchx();
	
	
}
//-----------------------------------------------------------------------------

void CursorXOR(void)
{
	TPoint AnglePos;
	
	AnglePos.x=30*cos((PI/180)*Game->DartAngle);
	AnglePos.y=30*sin((PI/180)*Game->DartAngle);
	
	Sprite8_XOR ((50+AnglePos.x)-4, (96-AnglePos.y)-4, 8, Image_Cursor, Plane0);
}
//-----------------------------------------------------------------------------

void RefreshScreen(void)
{
	
	
	FastCopyScreen(Plane1,Plane0);
	PortSet(Plane0,239,127);
	
	if (!Game->DartThrown)
	{
		Sprite8_OR(104,100-Game->DartSpeed,Game->DartSpeed,SpeedSprite,Plane0);
		CursorXOR();
	}
	
	PortRestore();
	FastCopyScreen(Plane0,LCD_MEM);
	
	
}
//--------------------------------------------------------------------------

void GetDartPos(void)
{
	
	float RealRangle=abs(Game->DartAngle-90)/2;
	
	
	Game->DistanceToTarget=50/cos((PI/180)*RealRangle);
	
	if (Game->DartAngle>90)
	Game->DartPos.x = 50-50*tan((PI/180)*RealRangle);
	else
	Game->DartPos.x = 50+50*tan((PI/180)*RealRangle);
	
	Game->DartPos.y = 100-(pow(Game->DartSpeed,2)/100);
	
	//Game->DartPos.x = random(100);
	//Game->DartPos.y = random(100);
	
	// BOUNDS
	
	if (Game->DartPos.x<4)
	Game->DartPos.x=4;
	if (Game->DartPos.y<4)
	Game->DartPos.y=4;
	
	if (Game->DartPos.x>96)
	Game->DartPos.x=96;
	if (Game->DartPos.y>96)
	Game->DartPos.y=96;
	
}
//--------------------------------------------------------------------------

void GetScore(void)
{
	
	// Game->CenterAngle
	
	if (Game->DartPos.x!=50)
	Game->CenterAngle = (180/PI)*atan((float)abs(50-Game->DartPos.y)/(float)abs(50-Game->DartPos.x));
	else
	{
		if (Game->DartPos.y<50)
		Game->CenterAngle = 90;
		else
		Game->CenterAngle = 270;
	}
	
	
	if (Game->DartPos.x>50 && Game->DartPos.y>50)
	Game->CenterAngle = 360-Game->CenterAngle;
	if (Game->DartPos.x<50 && Game->DartPos.y<50)
	Game->CenterAngle = 180-Game->CenterAngle;
	if (Game->DartPos.x<50 && Game->DartPos.y>50)
	Game->CenterAngle = 180+Game->CenterAngle;
	
	// Game->CenterDistance
	
	Game->CenterDistance = sqrt(pow((Game->DartPos.y-50),2)+pow((Game->DartPos.x-50),2));
	
	// Game->SpecialScore
	
	if (Game->CenterDistance<=3)
	Game->SpecialScore = S_BULLSEYE;
	else if (Game->CenterDistance>3 && Game->CenterDistance<=6)
	Game->SpecialScore = S_BULL;
	else if (Game->CenterDistance>6 && Game->CenterDistance<=30)
	Game->SpecialScore = S_DOUBLE;
	else if (Game->CenterDistance>30 && Game->CenterDistance<=50)
	Game->SpecialScore = S_SIMPLE;
	else
	Game->SpecialScore = S_MISSED;
	
	// Game->TargetPart
	Game->TargetPart = Game->CenterAngle/18;
	
	// Game->FinalScore
	Game->FinalScore=0;
	if (Game->SpecialScore==S_BULLSEYE)
	Game->FinalScore = 100;
	else if (Game->SpecialScore==S_BULL)
	Game->FinalScore = 50;
	else if (Game->SpecialScore!=S_MISSED)
	{
		Game->FinalScore = PointsList[Game->TargetPart];
		if (Game->SpecialScore==S_DOUBLE)
		Game->FinalScore*=2;
	}
	
	
}
//---------------------------------------------------------------------------

// Main Function
void _main(void)
{
	
	if (!(Plane0=malloc(2*LCD_SIZE)))
	goto end_prog;
	Plane1=Plane0+LCD_SIZE;
	
	TMenu MenuA;
	Menu = &MenuA;
	
	TGame GameA;
	Game=&GameA;
	
	//randomize();
	
	unsigned char SpeedSpriteA[100];
	SpeedSprite=SpeedSpriteA;
	memset(SpeedSprite,0xFF,100);
	
	clrscr();
	printf("Loading...");
	
	DrawTarget();
	
	for (;;)
	{
		
		MenuNew2();
		MenuAddTitle("Dart");
		MenuAddItem("Play");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		MenuDo(ME_MAIN);
		
		
		if (!Menu->ok) break;
		
		
		if (Menu->Index==1)
		{
			
			
			
			FontSetSys(0);
			
			
			HandlerFuncPtr5 = NULL;
			KB_Enable();
			WaitForAllKeysReleased();
			
			
			for(;;)
			{
				
				Game->DartAngle=90;
				Game->DartSpeed=50;
				Game->DartThrown=0;
				
				RefreshScreen();
				
				for(;;)
				{
					
					GetKeyboardState();
					
					if (kb_state&KEYFLAGS_LEFT)
					{
						Game->DartAngle+=6;
						if (Game->DartAngle>=180)
						Game->DartAngle=180;
						
						RefreshScreen();
					}
					else if (kb_state&KEYFLAGS_RIGHT)
					{
						Game->DartAngle-=6;
						if (Game->DartAngle<0)
						Game->DartAngle=0;
						
						RefreshScreen();
					}
					
					else if (kb_state&KEYFLAGS_UP)
					{
						Game->DartSpeed++;
						if (Game->DartSpeed>100)
						Game->DartSpeed=100;
						
						RefreshScreen();
					}
					else if (kb_state&KEYFLAGS_DOWN)
					{
						Game->DartSpeed--;
						if (Game->DartSpeed<0)
						Game->DartSpeed=0;
						
						RefreshScreen();
					}
					
					
					else if (kb_state_DOWN&KEYFLAGS_2ND)
					{
						break;
					}
					else if (kb_state&KEYFLAGS_ESC)
					goto end_loop;
					
					
				}
				
				WaitForAllKeysReleased();
				
				Game->DartThrown=1;
				GetDartPos();
				
				RefreshScreen();
				Sprite8_XOR (Game->DartPos.x-4,Game->DartPos.y-4, 8, Image_Cursor, LCD_MEM);
				
				
				GetScore();
				
				// STATS
				
				int stats_top=0;
				char buffer[16];
				
				sprintf(buffer,"Score : %d",Game->FinalScore);
				DrawStr(120,stats_top,buffer,A_NORMAL);
				stats_top+=10;
				
				DrawStr(120,stats_top,SpecialStr[Game->SpecialScore],A_NORMAL);
				stats_top+=10;
				
				
				WaitForKeyDOWN(KEYFLAGS_2ND);
				WaitForAllKeysReleased();
				
			}
			
			end_loop:
			
			WaitForAllKeysReleased();
			KB_Disable();
			
			FontSetSys(1);
			
		}
		
		
		else if (Menu->Index==2)
		{
			FontSetSys(0);
			clrscr();
			printf("About \n");
			printf("Dart 1.2 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
			
		}
		else
		break;
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
	
}


